! <ModelConstants_ml.f90 - A component of the EMEP MSC-W Unified Eulerian
!          Chemical transport Model>
!*****************************************************************************!
!*
!*  Copyright (C) 2007-2011 met.no
!*
!*  Contact information:
!*  Norwegian Meteorological Institute
!*  Box 43 Blindern
!*  0313 OSLO
!*  NORWAY
!*  email: emep.mscw@met.no
!*  http://www.emep.int
!*
!*    This program is free software: you can redistribute it and/or modify
!*    it under the terms of the GNU General Public License as published by
!*    the Free Software Foundation, either version 3 of the License, or
!*    (at your option) any later version.
!*
!*    This program is distributed in the hope that it will be useful,
!*    but WITHOUT ANY WARRANTY; without even the implied warranty of
!*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*    GNU General Public License for more details.
!*
!*    You should have received a copy of the GNU General Public License
!*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
!*****************************************************************************!
module ModelConstants_ml
 !+
 ! Specifies a number of constants used in the model. Note that
 ! physical constants (e.g. gravity, Cp, etc ( are specified in
 ! the module PhysicalConstants_ml.f90)
 !
 !----------------------------------------------------------------------------
use PhysicalConstants_ml, only : AVOG

implicit none
private

!=============================================================================
! Experiment name:
!  EMEPSTD      Standard run & output
!  EMEP2010    EMEPSTD with Iceland Volcanic Eruption input
!  TFMM        EMEPSTD, but with INERIS_SNAP & TFMM hourly output
!  FORECAST    Forecast run, MACC-ENS hourly output & BC
!  EVA2010     FORECAST with MACC-EVA2010 hourly output & BC
!  EMERGENCY   FORECAST with ONLY Volcanic Eruption & Nuclear Accident.
CHARACTER(LEN=*), public, parameter :: EXP_NAME="EMEP2010"

! FORECAST mode run:
! * Nested IC/BC def in Nest_ml & IFSMOZ_ExternalBICs_ml
! * Special hourly output def in My_Outputs_ml
! * Only dayly and hourly output are required,
!   all other output types to false in Derived_ml.
logical, public, parameter :: FORECAST=&
  (EXP_NAME=="FORECAST").or.(EXP_NAME=="EVA2010").or.(EXP_NAME=="EMERGENCY")

! Some flags for model setup
! will be removed when code is sufficiently tested
! (for convection use foundconv in permanent code)
logical, public, parameter ::         &
  USE_CONVECTION     = .false.,       & ! false works best for Euro runs,
  INERIS_SNAP1       = (EXP_NAME=="TFMM"), & ! Switches off decadal trend
  INERIS_SNAP2       = (EXP_NAME=="TFMM"), & ! Allows near-zero summer values
  USE_DEGREEDAY_FACTORS  = .true.,     & !
  USE_SOILWATER      = .true.,         & ! for deep soilwater,  under testing
  USE_FOREST_FIRES   = .false.,        & ! Needs global files, future
  USE_AIRCRAFT_EMIS  = .false.,        & ! Needs global file, see manual
  USE_LIGHTNING_EMIS = .true.,        & ! ok
  USE_SOILNOX        = .true.,        & ! ok, but diff for global + Euro runs
  NO_CROPNH3DEP      = .true.,        & ! Stop NH3 deposition for growing crops
  USE_SEASALT        = .true.,        & ! ok
! More experimental:
  USE_DUST           = .true.,       & ! Experimental
  USE_ROADDUST       = .false.,       & ! UNDER DEVELOPMENT! Testing the TNO Road Dust routine. So far with simplified "climate-correction" factor
  DO_SAHARA          = .true.,        & ! Turn on/off BG Saharan Dust
  USE_GLOBAL_SOILNOX = .false.,       & ! Need to design better switch
  USE_SOILNH3        = .false.,       & ! DUMMY VALUES, DO NOT USE!
  USE_AOD            = FORECAST,      &
  USE_ZREF           = .false.,       & ! testing
  USE_PFT_MAPS       = .false.,       & ! Future option
  EXTENDEDMASSBUDGET = .false.,       & ! extended massbudget outputs
  LANDIFY_MET        = .false.,       & ! extended massbudget outputs
  USE_POLLEN         = .false.,       & ! EXPERIMENTAL. Only works if start Jan 1
  USE_EMERGENCY      = .true.      ! Emergency: Volcanic Eruption & Nuclear Accident. Under development.

!Boundary layer profiles
  character(len=4), parameter, public :: FluxPROFILE = &
     "Iter"   !
!      "Ln95"   ! ! will use Launiainen1995  EXPERIMENTAL. Fails in some areas

! Biogenics. Use 3 even if no terpene chemistry - simplifies
! rest of code.  iso = isoprene, mtp = monoterpenes from pools,
! mtl = monoterpenes with light dependence
!DSA12 integer, public, parameter ::   NSOIL_EMIS = 2 ! NO + NH3
 integer, public, parameter ::   NBVOC = 3
 character(len=4),public, save, dimension(NBVOC) :: &
   BVOC_USED = (/ "Eiso","Emt ","Emtl"/)

!The GEA emission data, which is used for EUCAARI runs on the HIRHAM domains
!have in several sea grid cells non-zero emissions in other sectors than SNAP8
!and there are also NH3 emission over sea areas. The former problem makes
!the code crash if the sea areas are defined  as sea (sea=T), so we treat
!them as land in the EUCAARI/HIRHAM runs (sea=F). This is a problem with GEA
!emission data only, not the HIRHAM domain! When e.g. interpolated EMEP emissions
!are used on the HIRHAM domain, this is not a problem.

logical, public, parameter :: SEAFIX_GEA_NEEDED = .false. ! only if problems

!=============================================================================
!+ 1) Define first dimensions that might change quite often -  for different
!     run domains
character(len=*), parameter, public :: &
! DomainName = "EMEP-50kmEurope"
  DomainName = "EMEP-50kmEECCA"
! DomainName = "EMEP-1degGLOBAL"
! DomainName = "EMEPCWF-0.25degEurope"
! DomainName = "EMEPCWF-0.20degEurope"
! DomainName = "HIRHAM"

logical, parameter, public :: IS_GLOBAL = .false.

integer, public :: IIFULLDOM,JJFULLDOM!  & SET AUTOMATICALLY BY THE CODE
! IIFULLDOM = 182, JJFULLDOM = 197 ! x,y-Dimensions of full HIRHAM domain
! IIFULLDOM = 170, JJFULLDOM = 133 ! x,y-Dimensions of full EMEP domain
! IIFULLDOM = 132, JJFULLDOM = 159 ! x,y-Dimensions of full EECA domain
! IIFULLDOM = 840, JJFULLDOM = 832 ! x,y-Dimensions of full TNO07 domain     
! IIFULLDOM = 420, JJFULLDOM = 416 ! x,y-Dimensions of full TNO14 domain   
! IIFULLDOM = 210, JJFULLDOM = 208 ! x,y-Dimensions of full TNO28 domain 
! IIFULLDOM = 105, JJFULLDOM = 104 ! x,y-Dimensions of full TNO56 domain
! IIFULLDOM = 360, JJFULLDOM = 180 ! .... full GLOBAL domain
! IIFULLDOM = 201, JJFULLDOM = 161 ! .... full GEMS 0.25 domain
! IIFULLDOM = 301, JJFULLDOM = 221 ! .... full GEMS 0.25 extended domain
! IIFULLDOM = 321, JJFULLDOM = 221 ! .... full MACC 0.20 domain

! The difference between EMEP and EECCA is confusing...
integer, public, parameter :: &
! OFFSET_i=  0, OFFSET_j=  0    ! EMEP or default
  OFFSET_i=-35, OFFSET_j=-11    ! EECCA

integer, public, save, dimension(4) ::   &
!                 x0   x1  y0   y1
RUNDOMAIN = (/  -999,-999 ,  -999, -999 /)     ! Set values later
! RUNDOMAIN = (/  1, 182,  1, 197 /)     ! HIRHAM
! RUNDOMAIN = (/  1, 132,  1, 159 /)     ! EECCA = new EMEP domain
!  RUNDOMAIN = (/  1, 100,  1, 100 /)     ! Orig EMEP domain in EECCA (for benchmarks)
! RUNDOMAIN = (/ 40, 210, 12, 184 /)     ! SR TNO28 area
! RUNDOMAIN = (/  1, 210,  1, 208 /)     ! TNO28
! RUNDOMAIN = (/240, 720, 48, 736 /)     ! TNO07 reduced (15W-45E;30N-73N)
! RUNDOMAIN = (/120, 360, 24, 368 /)     ! TNO14 reduced (15W-45E;30N-73N)
! RUNDOMAIN = (/ 60, 180, 12, 184 /)     ! TNO28 reduced (15W-45E;30N-73N)
! RUNDOMAIN = (/ 70, 110, 72, 110 /)     ! TNO28  test
! RUNDOMAIN = (/ 30,  90,  6,  92 /)     ! TNO56 reduced (15W-45E;30N-73N)
! RUNDOMAIN = (/ 60, 180, 12, 184 /)     !  test TNO7 area
!--
! Suggestions, 6th June 2012, for TFMM_RUNS scale-dep -----------------------
! RUNDOMAIN = (/  40, 210, 12, 184 /)     ! TNO28 SR area (25W-60E;30N-73N)
! i.e.  - adds 20 squares west, 30 east to TNO28
! RUNDOMAIN = (/  160, 840, 48, 736 /)    ! TNO07  - add 80, 120
! RUNDOMAIN = (/   80, 420, 24, 368 /)    ! TNO14  - add 40, 60
! RUNDOMAIN = (/   20, 105, 6, 92 /)      ! TNO56  - add 10, 15
!----------------------------------------------------------------------------

! RUNDOMAIN = (/ 36, 167, 12, 122 /)     ! EMEP domain in PARLAM
! RUNDOMAIN = (/  1, 360,  1, 180 /)     ! FULL GLOBAL
! RUNDOMAIN = (/  1, 132,  1, 111 /)     ! EECCA, rep09
! RUNDOMAIN = (/  1, 132,  1, 159 /)     ! EECCA, rep10
! RUNDOMAIN = (/ 20, 167,  1, 122 /)     ! OSPAR/HELCOM domain
! RUNDOMAIN = (/ 18, 169,  1, 124 /)     ! OSPAR/HELCOM domain+borders
! RUNDOMAIN = (/  1, 201,  1, 161 /)     ! EMEP-CWF, GEMS 0.25 domain
! RUNDOMAIN = (/  1, 301, 26, 221 /)     ! EMEP-CWF, GEMS 0.25 extended domain
! RUNDOMAIN = (/  1, 321,  1, 221 /)     ! EMEP-CWF, MACC 0.20 domain
! RUNDOMAIN = (/ 70+OFFSET_i, 90+OFFSET_i, 43+OFFSET_j,  63+OFFSET_j /) ! (UK)
! RUNDOMAIN = (/ 60+OFFSET_i, 86+OFFSET_i, 43+OFFSET_j,  59+OFFSET_j /) ! (UK)
! RUNDOMAIN = (/ 85+OFFSET_i,120+OFFSET_i, 55+OFFSET_j,  70+OFFSET_j /) ! (changeable)
! RUNDOMAIN = (/ 85+OFFSET_i,120+OFFSET_i, 15+OFFSET_j,  50+OFFSET_j /) ! (changeable)
! RUNDOMAIN = (/ 75+OFFSET_i,110+OFFSET_i, 45+OFFSET_j,  60+OFFSET_j /) ! (gets Esk)
! RUNDOMAIN = (/ 80+OFFSET_i, 106+OFFSET_i, 33+OFFSET_j,  55+OFFSET_j /) ! (France)
! RUNDOMAIN = (/ 80+OFFSET_i, 106+OFFSET_i, 13+OFFSET_j,  35+OFFSET_j /) ! Southern domain
! RUNDOMAIN = (/ 75+OFFSET_i,110+OFFSET_i, 25+OFFSET_j,  60+OFFSET_j /) ! (gets Esk)

integer, public, save ::  & ! Actual number of processors in longitude, latitude
  NPROCX, NPROCY, NPROC     ! and total. NPROCY must be 2 for GLOBAL runs.

!=============================================================================
!+ 2) Define  debug flags.

! We have one variable, to say if we are on master-processor
! or not: (kept here to avoid too many dependencies for box-model
! codes which don't need Par_ml

logical, public, save ::  MasterProc = .true.
logical, public, save ::  DebugCell  = .false.

! For debugging, we often want to print out for  a specific location
! Set here:

! The coordinates given here only apply for the standard EMEP domain
integer, private, parameter :: &
!  DEBUG_ii= -99, DEBUG_jj= -99 ! none
! DEBUG_ii= 79, DEBUG_jj= 56 ! Eskdalemuir
! DEBUG_ii= 73, DEBUG_jj= 48 ! Mace Head
! DEBUG_ii= 88, DEBUG_jj= 53 ! Sibton
! DEBUG_ii= 88, DEBUG_jj= 53 ! Sibton
! DEBUG_ii= 91, DEBUG_jj= 71 ! Rorvik
! DEBUG_ii= 82, DEBUG_jj= 72 ! Voss, has some snow
! DEBUG_ii=110, DEBUG_jj= 48 ! High Vg!
! DEBUG_ii= 96, DEBUG_jj= 40 ! High VG_SO2_CF!
! DEBUG_ii=111, DEBUG_jj= 54 ! High VG_PMCO_CF!
! DEBUG_ii=101, DEBUG_jj= 51 ! Schauinsland
! DEBUG_ii=103, DEBUG_jj= 50 ! Mid-Europe
! DEBUG_ii= 93, DEBUG_jj= 57 ! Elspeetsche (52d12',5d45') 92.83, 56.64
! DEBUG_ii= 92, DEBUG_jj= 56 ! Cabauw
! DEBUG_ii= 97, DEBUG_jj= 62 ! Waldhof
! DEBUG_ii=116, DEBUG_jj= 63 ! K-Puszta
! DEBUG_ii=102, DEBUG_jj= 48 ! Payerne
! DEBUG_ii= 85, DEBUG_jj= 50 ! Harwell
! DEBUG_ii= 88, DEBUG_jj= 99 ! Harwell TNO TEST
! DEBUG_ii= 93, DEBUG_jj= 47 !  Grignon, France
! DEBUG_ii= 90, DEBUG_jj= 104 !  Wetland, Tundra
! DEBUG_ii= 72-OFFSET_i, DEBUG_jj= 37-OFFSET_j ! biomass burnung, Aug 2003
! DEBUG_ii= 90-OFFSET_i, DEBUG_jj= 27-OFFSET_j ! biomass burnung, Jul 2009
!DEBUG_ii= 58-OFFSET_i, DEBUG_jj= 72-OFFSET_j ! 99% water, SMI problems
!DUST DEBUG_ii= 94-OFFSET_i, DEBUG_jj= 24-OFFSET_j ! 99% water, dust problems
! DEBUG_ii= 85, DEBUG_jj= 35 ! Sea, Bay of Biscay
!DEBUG_ii= 76, DEBUG_jj= 65 ! Sea,  North sea
 DEBUG_ii= 66, DEBUG_jj= 50 ! Sea,  west UK
! DEBUG_ii= 80, DEBUG_jj= 52 ! Irish sea
! DEBUG_ii= 91, DEBUG_jj= 67 ! Tange
! DEBUG_ii=103, DEBUG_jj= 32 ! Prades, SMDge
! DEBUG_ii=128, DEBUG_jj= 13 !  Desert?

integer, public, parameter :: &
! DEBUG_i= 62, DEBUG_j= 45  ! SEA
  DEBUG_i= DEBUG_II+OFFSET_i, DEBUG_j= DEBUG_JJ+OFFSET_j    ! EMEP/EECCA
! DEBUG_i= 9, DEBUG_j= 201                                  ! MACC02
!  DEBUG_i= 0, DEBUG_j= 0    ! default

!=============================================================================
! Some flags for model setup

! Debug flag DEBUG_XXX  applied in subroutine XXX
 logical, public, parameter ::    &
   DEBUG_ADV            = .false. &
  ,DEBUG_AOT            = .false. &
  ,DEBUG_AQUEOUS        = .false. &
  ,DEBUG_BCS            = .false. &
  ,DEBUG_BIO            = .false. &
  ,DEBUG_BLM            = .false. & ! Produces matrix of differnt Kz and Hmix
  ,DEBUG_DERIVED        = .false. &
    ,DEBUG_COLUMN       = .false. & ! Extra option in Derived
  ,DEBUG_DO3SE          = .false. &
  ,DEBUG_DRYRUN         = .false. & ! Skips fast chemistry to save some CPU
  ,DEBUG_ECOSYSTEMS     = .false. &
  ,DEBUG_FORESTFIRE     = .false. &
  ,DEBUG_Kz             = .false. &
  ,DEBUG_MY_DERIVED     = .false. &
  ,DEBUG_DRYDEP         = .false. &
    ,DEBUG_VDS          = .false. &
    ,DEBUG_MY_DRYDEP    = .false. &
    ,DEBUG_CLOVER       = .false. &
    ,DEBUG_STOFLUX      = .false. &
  ,DEBUG_EMISSIONS      = .false. &
  ,DEBUG_EMISTIMEFACS   = .false. &
  ,DEBUG_EQUIB          = .false. &   !MARS, EQSAM etc.
  ,DEBUG_GETEMIS        = .false. &
  ,DEBUG_GRIDVALUES     = .false. &
  ,DEBUG_IOPROG         = .false. &
  ,DEBUG_LANDDEFS       = .false. &
  ,DEBUG_LANDUSE        = .false. &
  ,DEBUG_LANDPFTS       = .false. &
  ,DEBUG_LANDIFY        = .false. &
  ,DEBUG_MASS           = .false. &
  ,DEBUG_MET            = .false. &
  ,DEBUG_MOSAICS        = .false. &
  ,DEBUG_NEST           = .false. &
  ,DEBUG_NEST_ICBC      = .false. & ! IFS-MOZART BC
  ,DEBUG_NETCDF         = .false. &
  ,DEBUG_NETCDF_RF      = .false. & ! ReadField_CDF in NetCDF_ml
  ,DEBUG_NH3            = .false. & ! NH3Emis experimental
  ,DEBUG_OUTPUTCHEM     = .false. & ! Output of netcdf results
  ,DEBUG_OUT_HOUR       = .false. & ! Debug Output_hourly.f90
  ,DEBUG_pH             = .false. &
  ,DEBUG_PHYCHEM        = .false. &
  ,DEBUG_POLLEN         = .false.  &
  ,DEBUG_RUNCHEM        = .false. & ! DEBUG_RUNCHEM is SPECIAL
    ,DEBUG_AEROSOL      = .false. & ! ...needed for intended debugs are to work
    ,DEBUG_DUST           = .false. & ! Skips fast chemistry to save some CPU
    ,DEBUG_ROADDUST     = .false. &
    ,DEBUG_MY_WETDEP    = .false. &
    ,DEBUG_SEASALT      = .false. &
    ,DEBUG_SOA          = .false. &
    ,DEBUG_SOLVER       = .false. &
    ,DEBUG_SUBMET         = .false. &
    ,DEBUG_WETDEP       = .false. &
  ,DEBUG_RSUR           = .false. &
  ,DEBUG_RB             = .false. &
  ,DEBUG_SETUP_1DCHEM   = .false. &
  ,DEBUG_SETUP_1DBIO    = .false. &
  ,DEBUG_SITES          = .false. &
  ,DEBUG_SOILWATER      = .false. &
  ,DEBUG_SOILNOX        = .false. &
  ,DEBUG_VOLC           = .false. & ! Volcanoes
  ,DEBUG_EMERGENCY      = .false.    ! Emergency: Volcanic Eruption & Nuclear Accident. Under development.

!=============================================================================
! 3)  Source-receptor runs?
! We don't (generally) want daily outputs for SR runs, so in
! Derived_ml, we set all IOU_DAY false if SOURCE_RECPTOR = .true..

logical, public, parameter :: SOURCE_RECEPTOR = .false.

! Compress NetCDF output? (nc4 feature)
logical, public, parameter :: NETCDF_COMPRESS_OUTPUT=&
  (EXP_NAME/="FORECAST").and.(EXP_NAME/="EMERGENCY").and.(EXP_NAME/="3DPROFILES")
!logical, public, parameter :: NETCDF_COMPRESS_OUTPUT=.false.

!Hourly output in single file or monthly/daily files:
!NB: will not work well by default on Stallo per 14th Feb 2012 because of library bugs!
!Until this is fixed, you must compile with netcdf/4.1.3 and link and run with compiler 12.1.2
character(len=*), public, parameter :: &! ending depeding on date:
! HOURLYFILE_ending="_hour_YYYYMM.nc"   ! MM  -> month (01 .. 12)
! HOURLYFILE_ending="_hour_YYYYMMDD.nc" ! DD  -> day of the month (00 .. 31)
! HOURLYFILE_ending="_hour_YYYYJJJ.nc"  ! JJJ -> the day of the year (001 .. 366)
  HOURLYFILE_ending="_hour.nc"          ! keep the same for the whole run

! NH3 module as set up originally with U10 from met: kept for safety only.
! Will be replaced by sub.grid calculation of wind in future.
! Keep false until code re-implemented
logical, public, parameter :: NH3_U10 = .false.

! Nesting modes:
! produces netcdf dump of concentrations if wanted, or initialises mode runs
! from such a file. Used in Nest_ml:
!   0=donothing; 1=write; 2=read; 3=read and write;
!  10=write at end of run; 11=read at start; 12=read atstart and write at end (BIC)
integer, public, parameter ::NEST_MODE=0  

!=============================================================================
!+ 4)  Define main model dimensions,  things that will
!       generally only change when switching Met-driver
integer, public, parameter ::  &
  NLANDUSEMAX  = 23    &    ! Number of land use types in Inputs.Landuse file
, METSTEP      = 3     &    ! time-step of met. (h)
, KMAX_MID     = 20    &    ! Number of points (levels) in vertical
, KMAX_BND     = KMAX_MID+1 & ! Number of points (levels) in vertical + 1
, KTOP         = 1     &    ! K-value at top of domain
, KWINDTOP     = 5     &    ! Define extent needed for wind-speed array
, NMET         = 2     &    ! No. met fields in memory
, KCHEMTOP     = 2     &    ! chemistry not done for k=1
, KCLOUDTOP    = 8     &    ! limit of clouds (for MADE dj ??)
, KUPPER       = 6     &    ! limit of clouds (for wet dep.)
! And for My_Derived_ml VG_SPECS & LocalVariables_ml Vg_ref etc. we need a limit
, NVGOUT_MAX   = 10         ! Max. no species for My_Derived VG outputs


integer, public, parameter :: &   ! Groups for DDEP and WDEP
  SOX_INDEX = -1, OXN_INDEX = -2, RDN_INDEX = -3

! EMEP measurements end at 6am, used in  daily averages
integer, public, parameter :: END_OF_EMEPDAY  = 6

real, public, save :: dt_advec     ! time-step for advection (s)
real, public, save :: dt_advec_inv ! =1/dt_advec

! NTDAY:  Number of 2D O3 to be saved each day (for SOMO)
! 24/NTDAY is the time integration step for SOMO
! large value -> large memory use; too small value -> bad approx. for SOMO
! NB must be choosen:  24*3600/dt_advec <= NTDAY >=3 and
! preferably an integer fraction of 24*3600/dt_advec
integer, public, parameter :: NTDAY = 72

!/-- choose temperature range: from 148 K (-125C) ro 333K (+60C).
integer, parameter, public :: CHEMTMIN=148,CHEMTMAX=333

real, public, parameter :: &
  V_RAIN     = 5.           !approximate vertical speed of rain m/s

real, public, parameter :: &
  CW_THRESHOLD = 1.0E-7&!Cloudwater (kg/kg); above threshold allow possibility
                        ! for precipitations. Value could be adjusted.
, RH_THRESHOLD = 0.85  &!Relative humidity (fraction); above threshold allow
                        !possibility for precipitations.Value could be adjusted.
, CW2CC = 1.0E6         !Converts Cloudwater (kg/kg) into CloudCover in %
                        !Value could be adjusted.
!
!  additional parameters
!
integer, public, save   :: nterm, nmax, nstep, nprint, nass, nbound &
                         , iyr_trend ! Year specified for say BC changes

integer, public, save , dimension(20)   :: identi   !! ????
integer, public, parameter :: TXTLEN_NAME = 20
character(len=120), public, save :: runlabel1&!SHORT Allows explanatory text
                                  , runlabel2 !LONG  Read in from grun.pl

real, public, parameter :: &
  EPSIL  = 1.0e-30         &  ! small number
, PASCAL = 100.0           &  ! Conv. from hPa to Pa
, PPB    = 1.0e-9          &  ! parts per billion (mixing ratio)
, PPBINV = 1.0e+9          &
, PPT    = 1.0e-12         &  ! parts per trillion (mixing ratio)
, PPTINV = 1.0e+12         &
, PT     = 1.0e+4          &  ! Top of model region = 10000 Pa = 100 hPa
, Pref   = 101325.0        &  ! Reference pressure in Pa
, TINY   = 1.0e-9             ! -1.E-9" is sometimes used  in order to avoid
                              ! different roundings on different machines.

real, public, parameter :: &
  ATWAIR = 28.964          & ! Mol. weight of air (Jones, 1992)
, atwS   = 32.             & ! Atomic weight of Sulphur
, atwN   = 14.             & ! Atomic weight of Nitrogen
, atwPM  = 100.

! MFAC replaces earlier use of CHEFAC and ATWAIR - to scale from
! density (roa, kg/m3) to  molecules/cm3
! (kg/m3 = 1000 g/m3 = 0.001 * Avog/Atw molecules/cm3)
real, public, parameter :: MFAC = 0.001*AVOG/ATWAIR


! Define output types.
!   Derived output types: First 4 types (instantaneous,year,month,day),
!                         refer to output variables defined in Derived_ml.
!   Hourly  output types: Last 2 types (hourly inst.,hourly mean),
!                         refer to output variables defined in My_Outputs_ml.
!   IOU_HOUR_PREVIOUS: Auxiliary field for hourly accumulated Derived output
integer, public, parameter ::  &
  IOU_INST=1, IOU_YEAR=2, IOU_MON=3, IOU_DAY=4, & ! Derived output
  IOU_HOUR_PREVIOUS=5,                          & ! Aux. field
  IOU_HOUR=6, IOU_HOUR_MEAN=7                   & ! Hourly  output
  ,IOU_MAX_MAX=7                                  ! Max values for of IOU (for array declarations)

character(len=*), public, parameter :: model="EMEP_MSC-W"

end module ModelConstants_ml
!_____________________________________________________________________________
